"""
feather_creator

This package allows creating and saving feather files, from a motif database 
and gene sequences. It also allows mapping motif-TF interactions from 
a list of specific transcription factors.
"""
__author__ = "Timothée Frouté and Sergio Peignier"
__copyright__ = "Copyright 2019, The GReNaDIne Project"
__license__ = "GPL"
__version__ = "0.0.1"
__maintainer__ = "Sergio Peignier"
__email__ = "sergio.peignier@insa-lyon.fr"
__status__ = "pre-alpha"
